///////    Vector2D.java    ///////

/**
 * Clase para Vectores 2D
 *	Entregado para Tarea 2
 * @author A. Gonzalez */

public class Vector2D {

	/**
 	* Direcciones X e Y de vector
 	*/
	private double x, y;  // we will use cartesian coordinates
	
	/** 
 	* Constructor vacio  
	* 
	*/
	public Vector2D () {
		x = y = 0;
	}
	/** 
   * Constructor  con argumentos
 	* @param x 	magnitud eje X
 	* @param y 	magnitud eje Y
 	*/
	public Vector2D (double x, double y) {
		this.x = x;
		this.y = y;
	}
	/** 
 	* Obtencion de Coordenadas
	* @return the coordenada X	
 	*/
	public double getX(){
		return x;
	}
	/** 
 	* Obtencion de Coordenadas
	* @return coordenada Y	
 	*/
	public double getY(){
		return y;
	}
	/** 
 	* Seteo de Coordenadas X Y
 	* @param x 	magnitud eje X
 	* @param y 	magnitud eje Y
 	*/
	
	public void set(double x, double y){ // to set a new position
		this.x=x;
		this.y=y;
	}
	/** 
 	* Suma de Vectores
	* @param v 	Vector que suma
	* @return nuevo Vector2D sumado
 	*/
	public Vector2D plus(Vector2D v) {
		if (v==null) return new Vector2D(x,y);
		return new Vector2D(x+v.x, y+v.y);
	}
	/** 
 	* Multiplicacion por escalar
	* @param scalar Escalar por el que amplifica
	* @return Vector2D multiplicado
 	*/
	public Vector2D times(double scalar) {
		return new Vector2D(x*scalar, y*scalar);
	}
	/** 
 	* Resta de Vectores
	* @param v 	Vector que resta	
	* @return nuevo Vector2D restado
 	*/
	public Vector2D minus(Vector2D v) {
		if (v==null) return new Vector2D(x,y);
		return new Vector2D(x-v.x, y-v.y);
	}
	/** 
 	* Producto Punto de Vectores
	* @param v 	Vector Producto Punto
	* @return Valor Producto Punto
 	*/
	public double dotProduct(Vector2D v) {
		return x*v.x + y*v.y;
	}
	/** 
 	* Proyeccion en Vector
	* @param v Vector a Proyeccion
	* @return Vector Proyectado
 	*/
	public Vector2D getProjectionOn(Vector2D v){
		double newModule = x*v.x + y*v.y;
		return v.times(newModule/(v.x*v.x + v.y*v.y));
	}
	/** 
 	* Magnitud de Vector
	* @return Magnitud del Vector
 	*/
	public double module() {
		return Math.sqrt(x*x + y*y);
	}
	/** 
 	* Magnitud al Cuadrado
	* @return Magnitud cuadrada	
 	*/
	public double moduleSquared() {
		return x*x + y*y;
	}
	/** 
 	* Vector unitario
	* @return the new Vector Unitario
 	*/
	public Vector2D unitary() {
		return times(1/module());
	}	
	/** 
 	* Descripcion de Vector
	* @return String "(x,y)"
 	*/
	public static String getDescription() {
		return "(x,y)";
	}
	/** 
 	* Descripcion con Posicion
	* @return String con posiciones	
 	*/
	public String toString() {
		return "Vector "+x+","+y;
	}
}